// MurderSpot - a free game based on Clue/Cluedo
// Copyright 2005-2009 Christopher Elsby <chrise@chrise.me.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MURDERSPOT_BOARD_H
#define MURDERSPOT_BOARD_H

#include <wx/defs.h>
#include <wx/string.h>

#if 0
class MoveRule
{
public:
    MoveRule() { }
    ~MoveRule() {if(types != NULL) delete[] types;}
    // Square types being left, then square types being entered.
    // Each group is terminated by 255.
    // At least one of each group must match for the rule to apply.
    // Any type will match a group if is empty, or if the pointer is NULL.
    wxUint8 *types;
    // If this is true, the rule only applies if the 'from' and 'to' squares are adjacent.
    bool needadj;
    // Die points needed to move (index into Board::turnmap).
    wxUint8 movepoints;
    // Square type to "warp" to.
    wxUint8 warp;
};
#endif

struct StepResult
{
    // Die points needed to move (index into Board::turnmap).
    wxUint8 turnkey;
    // Square type to "warp" to.
    wxUint8 warptype;
};

class Board
{
public:
    // Constructor and destructor.
    Board();
    virtual ~Board() {unload();}
    // Loads a file.
    bool load(const wxString &file);
    // Unloads the data.
    void unload();
    // Sets the image file to the specified one, if it exists.
    void setImageFile(const wxString &imgfile, wxUint8 imgw = 0, wxUint8 imgh = 0);
    // Allocates space for the specified board dimensions and card counts.
    bool init(wxUint8 width, wxUint8 height, wxUint8 suspectcount, wxUint8 weaponcount, wxUint8 roomcount, wxUint8 suggestturns, wxUint8 accuseturns);
    // Gets the title of a data file.
    static wxString getTitle(const wxString &file);
    wxString getTitle() const {return title;}
    void setTitle(const wxString &newtitle) {title = newtitle;}
    // Indicates whether certain data is present.
    bool hasBoard() const {return board != NULL;}
    bool hasLocations() const {return locations != NULL;}
    // Cards.
    wxUint8 cardCount() const {return ncards;}
    wxString getNameSuspect() const {return sname;}
    bool setNameSuspect(const wxString &name) {sname = name; return TRUE;}
    wxString getNameWeapon() const {return wname;}
    bool setNameWeapon(const wxString &name) {wname = name; return TRUE;}
    wxString getNameRoom() const {return rname;}
    bool setNameRoom(const wxString &name) {rname = name; return TRUE;}
    wxUint8 suspectMin() const {return 0;}
    wxUint8 suspectMax() const {return offsetw;}
    wxUint8 suspectCount() const {return offsetw;}
    const wxString *suspectNames() const {return cards;}
    wxUint8 weaponMin() const {return offsetw;}
    wxUint8 weaponMax() const {return offsetr;}
    wxUint8 weaponCount() const {return offsetr - offsetw;}
    const wxString *weaponNames() const {if(cards == NULL) return NULL; return cards + offsetw;}
    wxUint8 roomMin() const {return offsetr;}
    wxUint8 roomMax() const {return ncards;}
    wxUint8 roomCount() const {return ncards - offsetr;}
    const wxString *roomNames() const {if(cards == NULL) return NULL; return cards + offsetr;}
    wxString getCardName(wxUint8 n) const {if(n < ncards) return cards[n]; return wxEmptyString;}
    bool setCardName(wxUint8 n, const wxString &name) {if(cards == NULL || n >= ncards) return FALSE; cards[n] = name; return TRUE;}
    // Board dimensions.
    wxUint8 getWidth() const {return w;}
    wxUint8 getHeight() const {return h;}
    // Gets the card for the room or suspect start that the specified square is in.
    wxUint8 getRoomAt(wxUint8 x, wxUint8 y) const {if(x < w && y < h && board[y * w + x] < ntypes) return types[board[y * w + x]]; return 255;}
    bool setRoomAt(wxUint8 x, wxUint8 y, wxUint8 card);
    // Roll the dice and return the result.
    wxUint8 getRoll() const;
    wxUint8 getMinRoll() const;
    wxUint8 getMaxRoll() const;
    wxString getDiceString() const;
    bool setDiceString(wxString str);
    // Die points needed.
    wxUint8 getSuggestTurns() const {return turnmap[63];}   // 63 == '?'
    void setSuggestTurns(wxUint8 turns) {turnmap[63] = turns;}
    wxUint8 getAccuseTurns() const {return turnmap[33];}    // 33 == '!'
    void setAccuseTurns(wxUint8 turns) {turnmap[33] = turns;}
    wxUint8 getMoveTurns(wxUint8 fromX, wxUint8 fromY, wxUint8 toX, wxUint8 toY, wxUint8 max = 254,
        wxUint8 *finalX = NULL, wxUint8 *finalY = NULL) const;
    wxUint8 getTurnConfigCount() const {return nturnconfig;}
    wxString getTurnConfigName(wxUint8 n) const
        {if(n >= nturnconfig || turnconfig == NULL) return wxEmptyString; return turnconfig[n].substr(1);}
    wxUint8 getTurnConfigValue(wxUint8 n) const
        {if(n >= nturnconfig || turnconfig == NULL || turnconfig[n].empty()) return 255; return turnmap[turnconfig[n].at(0)];}
    bool setTurnConfigValue(wxUint8 n, wxUint8 turns)
        {if(n >= nturnconfig || turnconfig == NULL || turnconfig[n].empty()) return FALSE; turnmap[turnconfig[n].at(0)] = turns; return TRUE;}
    // Suspect positions.
    wxUint8 getSuspectX(wxUint8 n) const {if(n < offsetw) return locations[n] % w; return 255;}
    wxUint8 getSuspectY(wxUint8 n) const {if(n < offsetw) return locations[n] / w; return 255;}
    wxUint8 getOriginalX(wxUint8 n) const {if(n < offsetw) return locations[n + offsetw] % w; return 255;}
    wxUint8 getOriginalY(wxUint8 n) const {if(n < offsetw) return locations[n + offsetw] / w; return 255;}
    wxUint8 getSuspectAt(wxUint8 x, wxUint8 y) const;
    wxUint8 getSuspectRoom(wxUint8 n) const {if(n < offsetw) return types[board[locations[n]]]; return 255;}
    wxUint8 getOriginalRoom(wxUint8 n) const {if(n < offsetw) return types[board[locations[n + offsetw]]]; return 255;}
    // Set suspect location.
    bool setSuspectPosition(wxUint8 n, wxUint8 x, wxUint8 y);
    // Commit the suspect's movement.
    bool commitMove(wxUint8 n) {if(n >= offsetw) return FALSE; locations[n + offsetw] = locations[n]; return TRUE;}
    // Cancel the suspect's movement.
    bool cancelMove(wxUint8 n) {if(n >= offsetw) return FALSE; locations[n] = locations[n + offsetw]; return TRUE;}
    // Determine whether the suspect has an unconfirmed move.
    bool isMoving(wxUint8 n) const {if(n >= offsetw) return FALSE; return locations[n] != locations[n + offsetw];}
    // Moves the suspect to the specified room (for when another player has suggested this suspect).
    bool roomWarp(wxUint8 suspect, wxUint8 room);
    // Board image path.
    wxString imageFile;
    // Image cell dimensions.
    wxUint8 imageWidth, imageHeight;
private:
    // Counts squares of the specified type(s).
    // If needempty is TRUE, only squares with no suspect on them will be counted.
    wxUint16 countSquares(wxUint8 type, bool needempty = TRUE) const {return countSquares(&type, 1, needempty);}
    wxUint16 countSquares(wxUint8 *matchtypes, wxUint8 typecount, bool needempty = TRUE) const;
#if 0
    wxUint16 findSquare(wxUint8 type, bool needempty = TRUE) const {return findSquare(&type, 1, needempty);}
    wxUint16 findSquare(wxUint8 *matchtypes, wxUint8 typecount, bool needempty = TRUE) const;
#endif
    // Gets a random square of the specified type.
    // If needempty is TRUE, only squares with no suspect on them will be returned.
    // (wxUint16) -1 is returned if no square was found.
    wxUint16 randSquare(wxUint8 type, bool needempty = TRUE) const {return randSquare(&type, 1, needempty);}
    wxUint16 randSquare(wxUint8 *matchtypes, wxUint8 typecount, bool needempty = TRUE) const;
    // Returns the result for a single-step move from one square to another.
    const StepResult *getStepResult(wxUint8 fromX, wxUint8 fromY, wxUint8 fromType, wxUint8 toX, wxUint8 toY, wxUint8 toType) const
        {return (((fromX == toX && (fromY == toY + 1 || toY == fromY + 1)) || (fromY == toY && (fromX == toX + 1 || toX == fromX + 1)))
            ? stepresultsadj : stepresults) + (fromType + ntypes * toType);}
#if 0
    // Checks the movement rules and returns a pointer to the rule with information on movement between the specified squares.
    // Where multiple matching rules exist, the last will be returned.
    // NULL is returned if no rule applies - then movepoints and warp should be taken to be 255.
    MoveRule *getMoveRule(wxUint8 x1, wxUint8 y1, wxUint8 x2, wxUint8 y2) const;
#endif
    // Title.
    wxString title;
    // Card group names.
    wxString sname; // e.g. "Suspects"
    wxString wname; // e.g. "Weapons"
    wxString rname; // e.g. "Rooms"
    // Card names. The array holds suspects, weapons and rooms, in that order.
    // offsetw is the number of suspects.
    // offsetr is the number of suspects + weapons.
    wxUint8 offsetw, offsetr, ncards;
    wxString *cards;
    // Suspect locations. Index is suspect card, value is (x + w * y).
    // The whole list is repeated, with the second copy only updated when the player's move is committed.
    wxUint16 *locations;
    // Number of square types.
    wxUint8 ntypes;
    // Mapping from square type to card.
    wxUint8 *types;
    // The board map. Indexed by (x + w * y). Value is square type.
    wxUint8 w, h;
    wxUint8 *board;
    // Die roll calculation - the first element is taken as is, and then random numbers from 0 to
    // each element minus 1 are added.
    wxUint8 ndice;
    wxUint8 *dice;
    // Map from square types to the result of an attempted movement.
    // Indexed by (startsquaretype + ntypes * endsquaretype).
    // stepresultsadj is used when the squares are adjacent, stepresults otherwise.
    StepResult *stepresults;
    StepResult *stepresultsadj;
#if 0
    // Movement rules.
    wxUint8 nmoverules;
    MoveRule *moverules;
#endif
    // Map from indentifier character to number of turns.
    // Indexed by character, value is turns.
    wxUint8 turnmap[256];
    // Configurable turnmap entries.
    // The first character of each entry is the turnmap index; the rest of the string is a description.
    wxUint8 nturnconfig;
    wxString *turnconfig;
};

#endif // !defined(MURDERSPOT_BOARD_H)
