// MurderSpot - a free game based on Clue/Cluedo
// Copyright 2005-2009 Christopher Elsby <chrise@chrise.me.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "global.h"

#include "logwin.h"
#include "cmdids.h"
#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/colour.h>
#include <wx/button.h>

LogWin::LogWin(wxWindow *parent) : wxPanel(parent)
{
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(log = new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY), 1, wxGROW);
#if 0 // Doesn't work very well
    log->SetBackgroundColour(*wxBLACK);
    log->SetForegroundColour(*wxWHITE);
    log->SetDefaultStyle(wxTextAttr(*wxWHITE, *wxBLACK));
#endif
    sizer->Add(new wxButton(this, ID_CLEARLOG, wxT("&Clear")), 0, wxALIGN_CENTER);
    SetSizerAndFit(sizer);
}

LogWin::~LogWin()
{
}

void LogWin::setLog(const wxString &msg)
{
    clear();
    addMessage(msg);
#if 0
    log->SetValue(msg);
    wxWindow *parent = GetParent();
    if(parent == NULL)
        Refresh();
    else
        parent->Refresh();
#endif
}

void LogWin::addMessage(const wxString &msg)
{
    log->AppendText(msg);
    wxWindow *parent = GetParent();
    if(parent == NULL)
        Refresh();
    else
        parent->Refresh();
}

void LogWin::clear()
{
    log->Clear();
}

#if 0
void LogWin::setWelcome()
{
    clear();
    addMessage(wxT(
        "Welcome to " PACKAGE_STRING "!\n"
        "MurderSpot comes with ABSOLUTELY NO WARRANTY. You use it entirely at your own risk.\n"
        "\n"
        "MurderSpot enables you to play on a single computer or over the Internet.\n"
        "\n"
        "To play a local game, with all players on this computer, click the \"Game\" menu at the top of the window and select"
        " \"Start Local Game\". This will start the game setup wizard, allowing you to pick the game type, settings and players.\n"
        "To see what cards you have, click \"Show Cards\" at the top of the window. Everyone needs to keep their own notes, either"
        " on a piece of paper or using a word processor.\n"
        "To move your character, click the square on the board that you want to go to. The status bar at the bottom of the window"
        " will show how many movement points you have left. All the important game actions are available as buttons at the top of"
        " the window, just below the menu bar.\n"
        "\n"
        "To play with others over the Internet, click the \"Network\" menu at the top of the window and select \"Connect\".\n"
        "Enter a nickname for yourself, and click OK to connect to the matchmaking server.\n"
        "Here you can meet and chat with other players. To talk with others, type a message in the box at the bottom of the window,"
        "and press enter to send it. Messages from others appear in the box above.\n"
        "To play a game, all players need to join a new room. To do this, click the \"Network\" menu at the top of the window and"
        " select \"Join room\". Pick a name for the room, for example \"#murderspot-mygame\". Note that you can switch between rooms"
        " by clicking the tabs at the top of the chat. To leave a room, click the X button, or click \"Close tab\" on the \"Network\""
        " menu. All chat room names begin with \"#\". The \"Network Status\" tab is special, and shows status messages from the game"
        " server. You cannot close this tab. When a new message is sent to a room, \"*\" appears at the end of its name. Click the tab"
        " to see the messages in that room.\n"
        "When all the players for the game have joined its room, the players should agree on the game type, settings, and who plays"
        " which character. The players should also agree on a host. The host is responsible for setting up the game. This player should"
        " click \"Host Network Game\" on the \"Game\" menu, after making sure the tab for the game's chat room is selected.\n"
        "This will start the game setup wizard. When the host completes the wizard, everyone else in the room will be asked whether"
        " they want to join. They should click \"Yes\". The game may take some time to initialise. When everyone sees the message"
        " announcing the first player's turn, the game is ready. Play proceeds as for a local game. You may continue to chat with the"
        " other players, but you should be careful not to reveal any information about your cards.\n"
        "\n"
        "If you have any questions about the game, feel free to ask in the matchmaking room.\n"
    ));
}
#endif
