// MurderSpot - a free game based on Clue/Cluedo
// Copyright 2005-2009 Christopher Elsby <chrise@chrise.me.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "global.h"

#include "mainwin.h"
#include "game.h"
#include "network.h"
#include "board.h"
#include "newnet.h"
#include "boardwin.h"
#include "chatwin.h"
#include "logwin.h"
#include "settingspage.h"
#include "cmdids.h"
#include <wx/frame.h>
#include <wx/event.h>
#include <wx/menu.h>
#include <wx/panel.h>
#include "cldnotebook.h"
#include <wx/sizer.h>
#include <wx/toolbar.h>
//#include <wx/accel.h>
#include <wx/wizard.h>
#include <wx/msgdlg.h>
#include <wx/textdlg.h>
#include <wx/choicdlg.h>
#include <wx/splitter.h>
#include <wx/statusbr.h>
#include <wx/log.h>
#include <wx/version.h>

// Returns something less than 0 if a is before b, 0 if they are the same, something greater than 0 if a is after b.
static int compareVersions(wxString a, wxString b)
{
    size_t pos;
    wxString sub;
    long aval, bval;
    while(!a.empty() || !b.empty())
    {
        pos = a.find(wxT("."));
        sub = a.substr(0, pos);
        aval = 0;
        sub.ToLong(&aval);
        if(pos < a.size())
            a.erase(0, pos + 1);
        else
            a.clear();
        pos = b.find(wxT("."));
        sub = b.substr(0, pos);
        bval = 0;
        sub.ToLong(&bval);
        if(pos < b.size())
            b.erase(0, pos + 1);
        else
            b.clear();
        if(aval != bval)
            return aval - bval;
    }
    return 0;
}

BEGIN_EVENT_TABLE(MainWin, wxFrame)
    EVT_MENU(-1, MainWin::OnCommand)
    EVT_BUTTON(-1, MainWin::OnCommand)
    EVT_UPDATE_UI(-1, MainWin::OnUpdateUI)
    EVT_CLOSE(MainWin::OnClose)
    MURDERSPOT_EVT_NOTEBOOK_PAGE_CLOSING(ID_CHATNOTEBOOK, MainWin::OnNotebookPageClosing)
END_EVENT_TABLE()

MainWin::MainWin() : wxFrame(NULL, -1, wxT("MurderSpot"), wxDefaultPosition, wxSize(800, 600)), alertBlock(0), updateStatus(TRUE),
    game(NULL), net(NULL), settingspage(NULL)
{
    // Menu bar.
    wxMenuBar *menubar = new wxMenuBar;
    // Game menu.
    wxMenu *menu = new wxMenu;
    menu->Append(ID_STARTLOCAL, wxT("Start &Local Game..."), wxT("Start a local game on this computer."));
    menu->Append(ID_SELECTTABLE, wxT("Ne&w Network Game..."), wxT("Start or join a network game with other players."));
    menu->AppendSeparator();
    menu->Append(ID_CHANGEIMAGE, wxT("Change Board &Image..."), wxT("Select a different board image for the active game,"
        " or change the size of the board."));
    menu->AppendSeparator();
    menu->Append(ID_SHOWPLAYERS, wxT("Show &Players..."), wxT("List the players in the game."));
    menu->Append(ID_SHOWCARDS, wxT("Show &Cards..."), wxT("List the cards in your hand."));
    menu->AppendSeparator();
    menu->Append(ID_CHANGEPLAYERS, wxT("Reassign Pla&yers..."), wxT("Change the players in the game."));
    menu->AppendSeparator();
    menu->Append(ID_CANCELMOVE, wxT("Cancel &Move"), wxT("Cancel your move, so that you can choose another."));
    menu->Append(ID_SUGGEST, wxT("&Suggest..."), wxT("Make a suggestion, for other players to disprove."));
    menu->Append(ID_DISPROVE, wxT("&Disprove..."), wxT("Disprove another player's suggestion."));
    menu->Append(ID_VIEWCARD, wxT("&View Card..."), wxT("See the card shown to disprove your suggestion."));
    menu->Append(ID_ACCUSE, wxT("&Accuse..."), wxT("Make your final accusation."));
    menu->Append(ID_ENDTURN, wxT("&End Turn"), wxT("End your turn, and let the next player make a move."));
    menu->AppendSeparator();
    menu->Append(ID_ENDGAME, wxT("Leave &Game..."), wxT("Leaves the current game, ending it if you are the host."));
    menu->Append(wxID_EXIT, wxT("E&xit..."), wxT("Exit MurderSpot, ending the game."));
    menubar->Append(menu, wxT("&Game"));
    // Network menu.
    menu = new wxMenu;
    menu->Append(ID_NETCONNECT, wxT("Co&nnect..."), wxT("Connect to a network server to play with others."));
    menu->Append(ID_NETDISCONNECT, wxT("&Disconnect"), wxT("Disconnect from the network."));
    menu->Append(ID_SELECTTABLE, wxT("Ne&w Network Game..."), wxT("Start or join a network game with other players."));
    menu->Append(ID_NETJOINCHANNEL, wxT("J&oin Chat Room..."), wxT("Join a chat room on the game server."));
    menu->Append(ID_NETPM, wxT("&Private Message..."), wxT("Starts a private conversation with another player."));
    menu->Append(ID_NETCLOSETAB, wxT("&Close Tab"), wxT("Close the current chat tab."));
    menubar->Append(menu, wxT("&Network"));
    // Help menu.
    menu = new wxMenu;
    menu->Append(ID_HELPABOUT, wxT("&About..."), wxT("Display the MurderSpot version number and copyright notice."));
    menubar->Append(menu, wxT("&Help"));
    SetMenuBar(menubar);
    // Tool bar.
    // TODO: Make some icons.
    wxToolBar *toolbar = CreateToolBar(wxTB_TEXT | wxTB_NOICONS);
    toolbar->AddTool(ID_ENDTURN, wxT("End Turn"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("End your turn, and let the next player make a move."));
    toolbar->AddTool(ID_SUGGEST, wxT("Suggest"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("Make a suggestion, for other players to disprove."));
    toolbar->AddTool(ID_DISPROVE, wxT("Disprove"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("Disprove another player's suggestion."));
    toolbar->AddTool(ID_VIEWCARD, wxT("View Card"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("See the card shown to disprove your suggestion."));
    toolbar->AddTool(ID_ACCUSE, wxT("Accuse"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("Make your final accusation."));
    toolbar->AddTool(ID_CANCELMOVE, wxT("Cancel"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("Cancel your move, so that you can choose another."));
    toolbar->AddSeparator();
    toolbar->AddTool(ID_SHOWCARDS, wxT("Cards"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("List the cards in your hand."));
    toolbar->AddTool(ID_SHOWPLAYERS, wxT("Players"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("List the players in the game."));
    toolbar->AddSeparator();
    toolbar->AddTool(ID_CHANGEPLAYERS, wxT("Reassign"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("Change the players in the game."));
    toolbar->AddSeparator();
    toolbar->AddTool(ID_ENDGAME, wxT("Leave"), wxNullBitmap, wxNullBitmap, wxITEM_NORMAL, wxEmptyString,
        wxT("Leaves the current game, ending it if you are the host."));
    toolbar->Realize();
    // Status bar.
    CreateStatusBar(1, 0, ID_STATUSBAR);
    // Main splitter window - board and game log in the top pane; chat in the bottom pane.
    splitter = new wxSplitterWindow(this, -1, wxDefaultPosition, wxDefaultSize, wxSP_3D | wxSP_LIVE_UPDATE);
    // Upper splitter window - board in the left pane; game log in the right pane.
    splitter2 = new wxSplitterWindow(splitter, -1, wxDefaultPosition, wxDefaultSize, wxSP_3D | wxSP_LIVE_UPDATE);
    // Welcome window.
//    welcomeWin = new WelcomeWin(splitter2);
    // Board window.
    boardWin = new BoardWin(splitter2, game);
    // Log window.
    logWin = new LogWin(splitter2);
//    logWin->setWelcome();
    // Notebook for chat.
    // wxNotebook is derived from wxControl, so I guess it's supposed to go on a panel...
    notebookpanel = new wxPanel(splitter);
    wxBoxSizer *notebookpanelsizer = new wxBoxSizer(wxVERTICAL);
    notebook = new Notebook(notebookpanel, ID_CHATNOTEBOOK);
    notebookpanelsizer->Add(notebook, 1, wxGROW);
    notebookpanel->SetSizerAndFit(notebookpanelsizer);
    // Network status window.
    statusWin = new ChatWin(notebook, this);
    notebook->AddPage(statusWin, wxT("Network Status"), FALSE);
    // Set up the initial display.
    splitter->Initialize(splitter2);
    splitter2->Initialize(boardWin); // Temporary - see newSettingsPage()
    notebookpanel->Hide();
    newSettingsPage(new WelcomePage);
}

MainWin::~MainWin()
{
    // Delete the game.
    if(game != NULL)
    {
        game->setManager(NULL);
        delete game;
        game = NULL;
    }
    // Delete the Network object.
    if(net != NULL)
    {
        net->mgr = NULL;
        delete net;
        net = NULL;
    }
}

void MainWin::newSettingsPage(SettingsPage *newpage)
{
    if(newpage != NULL)
        newpage->Create(this, splitter);
//    wxWindow *oldpane = settingspage == NULL ? (wxWindow*) boardWin : (wxWindow*) settingspage;
//    wxWindow *newpane = newpage == NULL ? (wxWindow*) boardWin : (wxWindow*) newpage;
    wxWindow *oldpane = settingspage == NULL ? (wxWindow*) splitter2 : (wxWindow*) settingspage;
    wxWindow *newpane = newpage == NULL ? (wxWindow*) splitter2 : (wxWindow*) newpage;
    if(oldpane == newpane)
        return;
    splitter->ReplaceWindow(oldpane, newpane);
//    newpane->SetSize(oldpane->GetSize());
    newpane->SetMinSize(wxSize(0, 0));
    oldpane->Hide();
    newpane->Show();
    if(settingspage != NULL)
        settingspage->Destroy();
    settingspage = newpage;
    // There are problems on Windows if this is done in the constructor...
    if(newpane == splitter2 && !splitter2->IsSplit())
    {
        splitter2->SplitVertically(boardWin, logWin);
        splitter2->SetMinimumPaneSize(10);
    }
    splitter->UpdateSize();
}

void MainWin::stdSettingsPage()
{
    if(!hasStdSettingsPage())
    {
        SettingsPage *newpage = NULL;
        if(game == NULL)
        {
            if(net == NULL)
                newpage = new WelcomePage;
            else
                newpage = new NetWelcomePage;
        }
        else if(game->getControlling())
        {
            if(!game->getBoard()->hasBoard())
                newpage = new GameDataFilePage;
            else if(!game->started())
                newpage = new GamePlayersPage;
        }
        else if(!game->started())
        {
            newpage = new WaitForHostPage;
        }
        newSettingsPage(newpage);
    }
}

bool MainWin::hasStdSettingsPage() const
{
    wxString type;
    if(game == NULL)
    {
        if(net == NULL)
            type = wxT("WelcomePage");
        else
            type = wxT("NetWelcomePage");
    }
    else if(game->getControlling())
    {
        if(!game->getBoard()->hasBoard())
            type = wxT("GameDataFilePage");
        else if(!game->started())
            type = wxT("GamePlayersPage");
    }
    else if(!game->started())
    {
        type = wxT("WaitForHostPage");
    }
    if(type.empty())
        return settingspage == NULL;
    else
        return settingspage != NULL && settingspage->getType() == type;
}

const NetChannel *MainWin::getChannel(int n) const
{
    if(net == NULL || notebook == NULL)
        return NULL;
    if(n < -1)
        n = notebook->GetSelection();
    if(n < 0 || n >= notebook->GetPageCount())
        return NULL;
    wxString text = notebook->GetPageText(n);
    if(text.empty() || text.at(0) != wxT('#'))
        return NULL;
    return net->getChannel(text.substr(0, text.find(wxT(" *"))));
}

void MainWin::receiveGameRequest(Network *netw, wxString gamid, const NetUser *sender, wxString type, wxString msg)
{
    if(netw == NULL || netw != net || sender == NULL || gamid.empty() || game == NULL || game->getGameID() != gamid)
        return;
    game->processNet(sender, type, msg);
}

void MainWin::receiveChat(Network *netw, wxString sender, wxString target, wxString type, wxString msg)
{
    if(netw == NULL || netw != net)
        return;
    bool isChan = !target.empty() && target.at(0) == wxT('#');
    type.MakeUpper();
    bool selectifnew = isChan || type == wxT("QUERY");
    // Send to the settings page.
    if(settingspage != NULL)
        settingspage->receiveChat(sender, target, type, msg);
#ifdef PACKAGE_VERSION
    // Alert the user if a new version of the program is available.
    if(type == wxT("TOPIC") && sender.empty() && target.Lower() == wxT("#murderspot"))
    {
        size_t endpos = msg.find(wxT(" :::: "));
        if(endpos > msg.size())
            endpos = msg.size();
        size_t pos = msg.find(wxT("Current version: "));
        if(pos < endpos)
        {
            pos += 17;
            size_t pos2 = msg.find(wxT(" "), pos);
            wxString currentver;
            currentver = msg.substr(pos, pos2 - pos);
            if(compareVersions(currentver, wxT(PACKAGE_VERSION)) > 0)
                wxLogMessage(wxT("A new version of MurderSpot is available.\nPlease visit http://murderspot.chrise.me.uk/ to obtain it."));
        }
    }
#endif
    // Look for a tab to send the message to.
    int pages = notebook->GetPageCount();
    int tab;
    wxString tabname;
    wxString matchname = (isChan || sender.empty() || (net != NULL && sender == net->getNick())) ? target : sender;
    if(!matchname.empty())
    {
        for(tab = 0; tab < pages; ++tab)
        {
            tabname = notebook->GetPageText(tab);
            tabname = tabname.substr(0, tabname.find(wxT(" *")));
            if(tabname.Lower() == matchname.Lower())
            {
                notebook->SetPageText(tab, ((type == wxT("NICK") && !isChan) ? msg : tabname) +
                    (tab == notebook->GetSelection() ? wxT("") : wxT(" *")));
                ChatWin *chat = (ChatWin*) notebook->GetPage(tab);
                chat->receiveChat(sender, target, type, msg);
                return;
            }
        }
    }
    // Send to status or create a new tab.
    if(matchname.empty() || type == wxT("MODE"))
    {
        for(tab = 0; tab < pages; ++tab)
        {
            tabname = notebook->GetPageText(tab);
            tabname = tabname.substr(0, tabname.find(wxT(" *")));
            if(tabname == wxT("Network Status"))
            {
                if(notebook->GetSelection() != tab)
                    notebook->SetPageText(tab, wxT("Network Status *"));
                break;
            }
        }
        statusWin->receiveChat(sender, target, type, msg);
    }
    else if(type != wxT("NICK") && type != wxT("PART") && type != wxT("QUIT"))
    {
        ChatWin *chat = new ChatWin(notebook, this);
        chat->sendto = matchname;
        notebook->AddPage(chat, matchname + (selectifnew ? wxT("") : wxT(" *")), selectifnew);
        chat->receiveChat(sender, target, type, msg);
    }
}

void MainWin::receiveNetStatus(Network *netw, wxString msg, bool showonall)
{
    if(netw == NULL || netw != net)
        return;
    wxString tabname;
    int pages = notebook->GetPageCount();
    int tab;
    ChatWin *chat;
    for(tab = 0; tab < pages; ++tab)
    {
        tabname = notebook->GetPageText(tab);
        tabname = tabname.substr(0, tabname.find(wxT(" *")));
        if(tabname == wxT("Network Status"))
        {
            if(notebook->GetSelection() != tab)
                notebook->SetPageText(tab, wxT("Network Status *"));
        }
        if(showonall)
        {
            chat = (ChatWin*) notebook->GetPage(tab);
            chat->receiveChat(wxEmptyString, wxEmptyString, wxT("STATUS"), msg);
        }
    }
    if(!showonall)
        statusWin->receiveChat(wxEmptyString, wxEmptyString, wxT("STATUS"), msg);
}

void MainWin::changeNick(Network *netw, wxString newnick)
{
    if(netw == NULL || netw != net)
        return;
    int pages = notebook->GetPageCount();
    for(int tab = 0; tab < pages; ++tab)
    {
        ChatWin *chat = (ChatWin*) notebook->GetPage(tab);
        chat->changeNick(newnick);
    }
    if(game != NULL && game->getNetwork() == netw)
        game->updateTableTopic();
}

void MainWin::addLog(const Game *gam, const wxString &msg)
{
    if(gam == game)
    {
        if(game != NULL && !game->getControlling() && settingspage != NULL && settingspage->getType() == wxT("WaitForHostPage"))
            newSettingsPage(NULL);
        logWin->addMessage(msg);
    }
}

void MainWin::clearLog(const Game *gam)
{
    if(gam == game)
        logWin->clear();
}

void MainWin::alert(const Game *gam, const wxString &msg)
{
    if(gam == game)
    {
        if(game != NULL && !game->getControlling() && settingspage != NULL && settingspage->getType() == wxT("WaitForHostPage"))
            newSettingsPage(NULL);
        // Note: wxMessageBox would block, so network users would not see the message
        // until the host dismissed the alert.
        if(alertBlock == 0 && game != NULL)
            wxLogMessage(wxT("%s"), msg.c_str());
    }
}

void MainWin::playerLost(const Game *gam)
{
    if(gam == game && settingspage == NULL)
        newSettingsPage(new GamePlayersPage);
}

void MainWin::setGame(Game *gam, bool updatesettingspage)
{
    if(gam == game)
        return;
    boardWin->setGame(gam);
    if(game != NULL)
    {
        game->setManager(NULL);
        game->stopGame();
        if(game->getChannel() != NULL)
            game->getChannel()->part();
        delete game;
    }
    game = gam;
    if(gam == NULL)
    {
        logWin->clear();
//        logWin->setWelcome();
    }
    else
    {
        gam->setManager(this);
        logWin->setLog(gam->getLog());
    }
    if(updatesettingspage)
        stdSettingsPage();
}

void MainWin::setNetwork(Network *newnet, bool updatesettingspage)
{
    if(newnet == net)
        return;
    if(net != NULL)
    {
        if(game != NULL && game->getNetwork() == net)
        {
            game->setChannel(NULL);
            setGame(NULL, FALSE);
            update();
        }
        delete net;
    }
    net = newnet;
    if(newnet != NULL)
    {
        newnet->mgr = this;
        if(!splitter->IsSplit())
        {
            notebookpanel->Show();
            splitter->SplitHorizontally(splitter->GetWindow1(), notebookpanel);
            splitter->SetMinimumPaneSize(10);
        }
    }
    if(updatesettingspage && game == NULL)
        stdSettingsPage();
}

void MainWin::receiveGameLeft(Network *netw, wxString gamid)
{
    if(netw == NULL || netw != net)
        return;
    if(game != NULL && game->getGameID() == gamid)
    {
        setGame(NULL);
        update();
    }
}

void MainWin::receiveUserLeft(const NetUser *user)
{
    if(game != NULL)
        game->removeNetUser(user);
}

void MainWin::receiveNetLost(Network *netw)
{
    if(game != NULL && game->getNetwork() == netw)
    {
        game->setChannel(NULL);
        setGame(NULL);
        update();
    }
}

void MainWin::update()
{
    updateStatus = TRUE;
    boardWin->Refresh();
}

void MainWin::update(const Game *gam)
{
    if(gam == game)
        update();
}

bool MainWin::updateImage()
{
    if(game != NULL && !game->getControlling() && settingspage != NULL && settingspage->getType() == wxT("WaitForHostPage"))
        newSettingsPage(NULL);
    return boardWin->updateImage();
}

void MainWin::updateImage(const Game *gam)
{
    if(gam == game)
        updateImage();
}

bool MainWin::confirmAbort()
{
    return game == NULL || game->canDiscard()
      || wxMessageBox(((game->getController() == NULL) ? wxT("Are you sure you want to end the current game?") :
      wxT("Are you sure you want to leave the current game?")), wxT("Abort Game"), wxYES_NO | wxICON_QUESTION, this) == wxYES;
}

#if 0
void MainWin::startGame(const NetChannel *chan)
{
    Game *gam = new Game;
    gam->setChannel(chan);
    wxString title;
    if(chan == NULL)
        title = wxT("Start Local Game");
    else
        title = wxT("Host Game in ") + chan->name;
    wxWizard *wiz = new wxWizard(this, -1, title);
    GameWizardData *page = new GameWizardData(wiz, gam);
    wiz->SetPageSize(page->getPageSize());
    if(wiz->RunWizard(page))
    {
        if(gam->netStartGame() && gam->logNotice(gam->getGameDescription(), TRUE, FALSE) && gam->startGame())
        {
            setGame(gam);
            update();
            updateImage();
            gam = NULL;
        }
        else
        {
            wxLogError(wxT("Failed to start game."));
        }
    }
    page->Destroy();
    wiz->Destroy();
    if(gam != NULL)
        delete gam;
}
#endif

void MainWin::changeImage()
{
    boardWin->changeImage();
#if 0
    wxWizard *wiz = new wxWizard(this, -1, wxT("Change Image"));
    GameWizardImage *page = new GameWizardImage(wiz, game);
    wiz->SetPageSize(page->getPageSize());
    if(wiz->RunWizard(page))
        updateImage();
    page->Destroy();
    wiz->Destroy();
#endif
}

bool MainWin::allowCloseTab(int n) const
{
    if(n < -1)
        n = notebook->GetSelection();
    if(n < 0 || n >= notebook->GetPageCount())
        return FALSE;
    wxString tabname = notebook->GetPageText(n);
    tabname = tabname.substr(0, tabname.find(wxT(" *")));
    if(tabname == wxT("Network Status"))
        return FALSE;
    return TRUE;
}

bool MainWin::confirmCloseTab(int n)
{
    if(!allowCloseTab(n))
        return FALSE;
    if(tabclosenoconfirm > 0)
        return TRUE;
    const NetChannel *chan = getChannel(n);
    return chan == NULL || game == NULL || game->getChannel() != chan || confirmAbort();
}

void MainWin::OnCommand(wxCommandEvent &event)
{
    ++alertBlock;
    switch(event.GetId())
    {
    case ID_NETCONNECT:
        if(game == NULL || net == NULL || game->getNetwork() != net || confirmAbort())
        {
            NetClientDlg dlg(this);
            Network *newnet = dlg.runDialog();
            if(newnet != NULL)
            {
                setNetwork(newnet);
                receiveNetStatus(newnet, wxT("Connecting..."));
            }
        }
        break;
    case ID_NETDISCONNECT:
        if(net != NULL && (game == NULL || game->getNetwork() != net || confirmAbort()))
        {
            net->disconnect();
            setNetwork(NULL);
        }
        break;
    case ID_NETJOINCHANNEL:
        if(net != NULL)
        {
            wxString channame = wxGetTextFromUser(wxT("Which room do you want to join?"), wxT("Join Room"), wxT(""), this);
            if(!channame.empty())
            {
                if(channame.at(0) != wxT('#'))
                    channame.insert(0, wxT("#"));
                net->join(channame);
            }
        }
        break;
    case ID_NETPM:
        if(net != NULL)
        {
            wxString target = wxGetTextFromUser(wxT("Which user do you want to start a conversation with?"),
                wxT("Private Message"), wxT(""), this);
            if(!target.empty())
                receiveChat(net, target, net->getNick(), wxT("QUERY"), wxT(""));
        }
        break;
    case ID_NETCLOSETAB:
        {
        int sel = notebook->GetSelection();
        if(confirmCloseTab(sel))
        {
            if(game != NULL && game->getChannel() == getChannel(sel))
            {
                setGame(NULL);
                update();
            }
            ++tabclosenoconfirm;
            notebook->DeletePage(sel);
            --tabclosenoconfirm;
        }
        }
        break;
    case ID_STARTLOCAL:
        if(confirmAbort())
        {
            setGame(new Game);
            update();
        }
        break;
    case ID_TABLEHOST:
        if(net != NULL && net->connected() && tablelist != NULL && tablelist->canHost() && confirmAbort())
        {
            wxString chan = tablelist->getSelectedChannel();
            wxString desc = tablelist->getDescription();
            wxString psw = tablelist->getPassword();
            // We always use a channel key to prevent more than one user accidentally hosting in the same channel.
            if(psw.empty())
                psw = wxT("murderspot");
            net->command(wxT("JOIN ") + chan);
            net->command(wxT("MODE ") + chan + wxT(" +ntk :") + psw);
            Game *gam = new Game;
            gam->setChannel(net->getChannel(chan));
            gam->makeHost();
            gam->setDescription(desc, FALSE);
            gam->updateTableTopic();
            setGame(gam);
            update();
        }
        break;
    case ID_TABLEJOIN:
        if(net != NULL && net->connected() && tablelist != NULL && tablelist->canJoin() && confirmAbort())
        {
            wxString chan = tablelist->getSelectedChannel();
            wxString psw = tablelist->getPassword();
            if(psw.empty())
                psw = wxT("murderspot");
            net->command(wxT("JOIN ") + chan + wxT(" :") + psw);
            Game *gam = new Game;
            gam->setChannel(net->getChannel(chan));
            gam->makeGuest(net->getUser(tablelist->getSelectedHost()));
            setGame(gam);
            update();
        }
        break;
    case ID_SELECTTABLE:
        if(net != NULL && net->connected() && confirmAbort())
        {
            setGame(NULL, FALSE);
            newSettingsPage(new SelectTablePage);
        }
        // Intentional run-on.
    case ID_TABLEGETLIST:
        if(net != NULL && net->connected() && tablelist != NULL)
        {
            tablelist->beginTableList();
            net->command(wxT("LIST #murderspot-table*"));
        }
        break;
    case wxID_CANCEL:
        stdSettingsPage();
        break;
    case ID_CHANGEIMAGE:
        if(game != NULL && game->started())
            changeImage();
        break;
    case ID_CLEARLOG:
        if(game != NULL)
            game->clearLog();
    case ID_CANCELMOVE:
        if(game != NULL && game->canControl())
        {
            game->cancelMove();
            game->update();
        }
        break;
    case ID_SUGGEST:
        if(game != NULL && game->canControl() && game->canSuggest())
        {
            Board *board = game->getBoard();
            int suspect = wxGetSingleChoiceIndex(board->getNameSuspect() + wxT(":"), wxT("Make Suggestion"), board->suspectCount(), board->suspectNames(), this) + board->suspectMin();
            if(suspect < board->suspectMin() || suspect >= board->suspectMax())
                break;
            int weapon = wxGetSingleChoiceIndex(board->getNameWeapon() + wxT(":"), wxT("Make Suggestion"), board->weaponCount(), board->weaponNames(), this) + board->weaponMin();
            if(weapon < board->weaponMin() || weapon >= board->weaponMax())
                break;
            int room = game->getBoard()->getSuspectRoom(game->getActive());
            if(room < board->roomMin() || room >= board->roomMax())
                break;
            if(wxMessageBox(wxT("Are you sure you want to make this suggestion?\n") + board->getCardName(suspect) + wxT("\n")
                        + board->getCardName(weapon) + wxT("\n") + board->getCardName(room), wxT("Make Suggestion"), wxYES_NO | wxICON_QUESTION, this) == wxYES)
            {
                game->suggest(suspect, weapon);
                game->update();
            }
        }
        break;
    case ID_DISPROVE:
        if(game != NULL && game->canControl(game->getDisprover()) && game->canDisprove())
        {
            Board *board = game->getBoard();
            wxString choices[3];
            wxUint8 cards[3];
            int n = 0;
            if(game->needHideCards() && wxMessageBox(board->getCardName(game->getDisprover()) +
              wxT(" needs to pick a card.\nMake sure everyone else is looking away!"), wxT("Disprove Suggestion"), wxOK | wxCANCEL, this) != wxOK)
                break;
            if(game->getCardHolder(game->suggestedSuspect()) == game->getDisprover())
            {
                choices[n] = board->getCardName(game->suggestedSuspect());
                cards[n] = game->suggestedSuspect();
                ++n;
            }
            if(game->getCardHolder(game->suggestedWeapon()) == game->getDisprover())
            {
                choices[n] = board->getCardName(game->suggestedWeapon());
                cards[n] = game->suggestedWeapon();
                ++n;
            }
            if(game->getCardHolder(game->suggestedRoom()) == game->getDisprover())
            {
                choices[n] = board->getCardName(game->suggestedRoom());
                cards[n] = game->suggestedRoom();
                ++n;
            }
            n = wxGetSingleChoiceIndex(wxT("Which card do you want to show ") + board->getCardName(game->getActive()) + wxT("?"),
                wxT("Disprove Suggestion"), n, choices, this);
            if(n > -1 && n < 3)
            {
                game->disprove(cards[n]);
                game->update();
            }
        }
        break;
    case ID_VIEWCARD:
        if(game != NULL && game->canControl() && game->hasDisproved())
        {
            if(game->needHideCurrent() && wxMessageBox(game->getBoard()->getCardName(game->getActive()) + wxT(" needs to see ") +
              game->getBoard()->getCardName(game->getDisprover()) + wxT("'s card.\nMake sure everyone else is looking away!"),
              wxT("View Card"), wxOK | wxCANCEL, this) != wxOK)
                return;
            wxMessageBox(game->getBoard()->getCardName(game->getDisprover()) + wxT(" has shown ") + game->getBoard()->getCardName(game->getActive()) +
              wxT(" the following card:\n") + game->getBoard()->getCardName(game->disprovedCard()) + wxT("\n\nDon't forget to click Accuse or End Turn"
              " when you are ready!"), wxT("View Card"), wxOK, this);
        }
        break;
    case ID_ACCUSE:
        if(game != NULL && game->canControl() && game->canAccuse())
        {
            Board *board = game->getBoard();
            int suspect = wxGetSingleChoiceIndex(board->getNameSuspect() + wxT(":"), wxT("Make Accusation"), board->suspectCount(), board->suspectNames(), this) + board->suspectMin();
            if(suspect < board->suspectMin() || suspect >= board->suspectMax())
                break;
            int weapon = wxGetSingleChoiceIndex(board->getNameWeapon() + wxT(":"), wxT("Make Accusation"), board->weaponCount(), board->weaponNames(), this) + board->weaponMin();
            if(weapon < board->weaponMin() || weapon >= board->weaponMax())
                break;
            int room = wxGetSingleChoiceIndex(board->getNameRoom() + wxT(":"), wxT("Make Accusation"), board->roomCount(), board->roomNames(), this) + board->roomMin();
            if(room < board->roomMin() || room >= board->roomMax())
                break;
            if(wxMessageBox(wxT("Are you sure you want to make this accusation?\n")
                    + board->getCardName(suspect) + wxT("\n") + board->getCardName(weapon) + wxT("\n") + board->getCardName(room),
                    wxT("Make Accusation"), wxYES_NO | wxICON_QUESTION, this) == wxYES
                && wxMessageBox(wxT("WARNING: This is your final accusation.\nIf it is wrong, you will LOSE THE GAME!\n"
                    "If this is not what you were expecting, you probably intended to click Suggest instead of Accuse.\n"
                    "Only click OK if you absolutely sure that you are ready to make your final accusation."),
                    wxT("Final Warning"), wxOK | wxCANCEL | wxICON_EXCLAMATION, this) == wxOK)
            {
                game->accuse(suspect, weapon, room);
                game->update();
            }
        }
        break;
    case ID_ENDTURN:
        if(game != NULL && game->inProgress() && game->canControl() && !game->activeSuggestion())
        {
            game->endTurn();
            game->update();
        }
        break;
    case ID_ENDGAME:
        if(confirmAbort())
        {
            if(game != NULL)
                game->stopGame();
            setGame(NULL);
            update();
        }
        break;
    case ID_SHOWPLAYERS:
        if(game != NULL && game->started())
        {
            wxString msg = wxT("The following players are in the game:");
            Player *pl;
            for(wxUint8 n = 0; n < game->getBoard()->suspectMax(); ++n)
            {
                pl = game->getPlayer(n);
                if(pl != NULL && pl->type > 0 && pl->type < playerTypeCount)
                {
                    msg += wxT("\n") + game->getBoard()->getCardName(n) + wxT(" is played by ");
                    if(pl->type > 1)
                    {
                        msg += playerTypeNames[pl->type];
                    }
                    else if(pl->name.empty())
                    {
                        if(pl->netuser == NULL)
                            msg += wxT("Local player");
                        else
                            msg += wxT("Network player ") + pl->netuser->nick;
                    }
                    else
                    {
                        msg += pl->name;
                        if(pl->netuser == NULL)
                            msg += wxT(" (Local player)");
                        else
                            msg += wxT(" (Network player ") + pl->netuser->nick + wxT(")");
                    }
                }
            }
            wxMessageBox(msg, wxT("Show Players"), wxOK, this);
        }
        break;
    case ID_SHOWCARDS:
        if(game != NULL && game->started())
        {
            wxUint8 choices[game->getBoard()->suspectCount() + 1];
            wxString names[game->getBoard()->suspectCount() + 1];
            wxUint8 choicecount = 1;
            choices[0] = 255;
            names[0] = wxT("All Cards");
            wxUint8 card;
            for(card = game->getBoard()->suspectMin(); card < game->getBoard()->suspectMax(); ++card)
            {
                if(game->canControl(card))
                {
                    choices[choicecount] = card;
                    names[choicecount] = game->getBoard()->getCardName(card);
                    ++choicecount;
                }
            }
            if(choicecount == 0)
                break;
            wxString msg = wxT("Choose which player's cards to view.");
            if(choicecount > 1)
                msg += wxT("\nMake sure everyone else is looking away!");
//            {
                int choice = wxGetSingleChoiceIndex(msg, wxT("Show Cards"), choicecount, names, this);
                if(choice < 0 || choice >= choicecount)
                    break;
                choices[0] = choices[choice];
//            }
            if(choices[0] == 255)
                msg = wxT("The following cards are in the game:");
            else
                msg = game->getBoard()->getCardName(choices[0]) + wxT(" has the following cards:");
            for(card = 0; card < game->getBoard()->cardCount(); ++card)
            {
                if(card == game->getBoard()->suspectMin())
                    msg += wxT("\n\n") + game->getBoard()->getNameSuspect() + wxT(":");
                else if(card == game->getBoard()->weaponMin())
                    msg += wxT("\n\n") + game->getBoard()->getNameWeapon() + wxT(":");
                else if(card == game->getBoard()->roomMin())
                    msg += wxT("\n\n") + game->getBoard()->getNameRoom() + wxT(":");
                if(choices[0] == 255 || game->getCardHolder(card) == choices[0])
                    msg += wxT("\n") + game->getBoard()->getCardName(card);
            }
            wxMessageBox(msg, wxT("Show Cards"), wxOK, this);
        }
        break;
    case ID_CHANGEPLAYERS:
        if(game != NULL && game->started() && game->getControlling() && settingspage == NULL)
            newSettingsPage(new GamePlayersPage);
        break;
    case ID_HELPABOUT:
        wxMessageBox(wxT(
            PACKAGE_STRING
            "\nCopyright 2005-2009 Christopher Elsby\nCompiled with ") wxVERSION_STRING wxT("\n"
            "This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public"
            " License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any"
            " later version.\n"
            "This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied"
            " warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more"
            " details.\n"
            "You should have received a copy of the GNU General Public"
            " License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth"
            " Floor, Boston, MA 02110-1301 USA."), wxT("About MurderSpot"), wxOK, this);
        break;
    case wxID_EXIT:
        --alertBlock;
        Close();
        return;
    default:
        event.Skip();
    }
    --alertBlock;
}

void MainWin::OnUpdateUI(wxUpdateUIEvent &event)
{
    switch(event.GetId())
    {
    case ID_SELECTTABLE:
    case ID_NETJOINCHANNEL:
    case ID_NETPM:
        event.Enable(net != NULL && net->connected());
        break;
    case ID_NETDISCONNECT:
        event.Enable(net != NULL);
        break;
    case ID_NETCLOSETAB:
        event.Enable(allowCloseTab());
        break;
    case ID_TABLEHOST:
        event.Enable(net != NULL && net->connected() && tablelist != NULL && tablelist->canHost());
        break;
    case ID_TABLEJOIN:
        event.Enable(net != NULL && net->connected() && tablelist != NULL && tablelist->canJoin());
        break;
    case ID_TABLEGETLIST:
        event.Enable(net != NULL && net->connected() && tablelist != NULL);
        break;
    case wxID_CANCEL:
        event.Enable(!hasStdSettingsPage());
        break;
    case ID_CHANGEIMAGE:
        event.Enable(game != NULL && game->started());
        break;
    case ID_CANCELMOVE:
        event.Enable(game != NULL && !game->isPaused() && game->canControl() && game->isMoving());
        break;
    case ID_SUGGEST:
        event.Enable(game != NULL && !game->isPaused() && game->canControl() && game->canSuggest());
        break;
    case ID_DISPROVE:
        event.Enable(game != NULL && !game->isPaused() && game->canControl(game->getDisprover()) && game->canDisprove());
        break;
    case ID_VIEWCARD:
        event.Enable(game != NULL && !game->isPaused() && game->canControl() && game->hasDisproved());
        break;
    case ID_ACCUSE:
        event.Enable(game != NULL && !game->isPaused() && game->canControl() && game->canAccuse());
        break;
    case ID_ENDTURN:
        event.Enable(game != NULL && !game->isPaused() && game->inProgress() && game->canControl() && !game->activeSuggestion());
        break;
    case ID_ENDGAME:
        event.Enable(game != NULL);
        break;
    case ID_SHOWPLAYERS:
        event.Enable(game != NULL && game->started());
        break;
    case ID_SHOWCARDS:
#if 0 // Was there a reason for this way of doing it??
        event.Enable(FALSE);
        if(game != NULL && !game->isPaused() && game->started() && game->getBoard()->cardCount() > 0)
            event.Enable(TRUE);
#endif
        event.Enable(game != NULL && !game->isPaused() && game->started() && game->getBoard()->cardCount() > 0);
        break;
    case ID_CHANGEPLAYERS:
        event.Enable(game != NULL && game->started() && game->getControlling() && settingspage == NULL);
        break;
    case ID_STATUSBAR:
        {
        wxStatusBar *statusbar;
        statusbar = (wxStatusBar*) event.GetEventObject();
        // We use the "MurderSpot: " prefix to distinguish the status message from anything pushed on.
        if(statusbar != NULL && (statusbar->GetStatusText().empty() ||
            (statusbar->GetStatusText().substr(0, 12) == wxT("MurderSpot: "))))
        {
            wxString newtext = (game == NULL ? wxT("MurderSpot: No game in progress") : (wxT("MurderSpot: ") + game->getStatusText()));
            if(newtext != statusbar->GetStatusText())
                statusbar->SetStatusText(newtext);
            updateStatus = FALSE;
        }
        break;
        }
    case ID_CHATNOTEBOOK:
        {
        int tab = notebook->GetSelection();
        if(tab >= 0)
        {
            wxString text = notebook->GetPageText(tab);
            size_t n = text.find(wxT(" *"));
            if(n < text.size())
                notebook->SetPageText(tab, text.substr(0, n));
        }
        }
    default:
        event.Skip();
    }
}

void MainWin::OnClose(wxCloseEvent &event)
{
    ++alertBlock;
    if(event.CanVeto() && !confirmAbort())
    {
        --alertBlock;
        event.Veto();
    }
    else
        Destroy();
}

void MainWin::OnNotebookPageClosing(NotebookEvent &event)
{
    if(!confirmCloseTab(event.GetSelection()))
        event.Veto();
}
