// MurderSpot - a free game based on Clue/Cluedo
// Copyright 2005-2009 Christopher Elsby <chrise@chrise.me.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MURDERSPOT_MAINWIN_H
#define MURDERSPOT_MAINWIN_H

#include <wx/frame.h>
#include <wx/event.h>

#include "manager.h"
#include "cldnotebookfwd.h"

class Game;
class Network;
class NetUser;
class NetChannel;
class BoardWin;
class LogWin;
class ChatWin;
//class WelcomeWin;
class SettingsPage;
class SelectTablePage;
class wxPanel;
class wxSplitterWindow;

class MainWin : public wxFrame, public Manager
{
public:
    MainWin();
    virtual ~MainWin();
    void update();
    virtual void update(const Game *gam);
    virtual void updateImage(const Game *gam);
    bool updateImage();
    bool confirmAbort();
    bool allowCloseTab(int n = -2) const;
    bool confirmCloseTab(int n = -2);
    Game *getGame() {return game;}
    const Game *getGame() const {return game;}
    void setGame(Game *gam, bool updatesettingspage = TRUE);
    Network *getNetwork() {return net;}
    const Network *getNetwork() const {return net;}
    void setNetwork(Network *newnet, bool updatesettingspage = TRUE);
    BoardWin *getBoardWin() {return boardWin;}
    const BoardWin *getBoardWin() const {return boardWin;}
    SelectTablePage *getTableList() {return tablelist;}
    const SelectTablePage *getTableList() const {return tablelist;}
    void setTableList(SelectTablePage *newval) {tablelist = newval;}
//    void startGame(const NetChannel *chan = NULL);
    void changeImage();
    const NetChannel *getChannel(int n = -2) const;
    virtual void receiveGameLeft(Network *netw, wxString gamid);
    virtual void receiveUserLeft(const NetUser *user);
    virtual void receiveNetLost(Network *netw);
    virtual void receiveGameRequest(Network *netw, wxString gamid, const NetUser *sender, wxString type, wxString msg);
    virtual void receiveChat(Network *netw, wxString sender, wxString target, wxString type, wxString msg);
    virtual void receiveNetStatus(Network *netw, wxString rawmsg, bool showonall = FALSE);
    virtual void changeNick(Network *netw, wxString newnick);
    virtual void addLog(const Game *gam, const wxString &msg);
    virtual void clearLog(const Game *gam);
    virtual void alert(const Game *gam, const wxString &msg);
    virtual void playerLost(const Game *gam);
    void newSettingsPage(SettingsPage *newpage);
    void stdSettingsPage();
    bool hasStdSettingsPage() const;
    int alertBlock;
private:
    bool updateStatus;
    int tabclosenoconfirm;
    Game *game;
    Network *net;
    BoardWin *boardWin;
    LogWin *logWin;
    ChatWin *statusWin;
    wxPanel *notebookpanel;
    Notebook *notebook;
    SettingsPage *settingspage;
    SelectTablePage *tablelist;
    wxSplitterWindow *splitter;
    wxSplitterWindow *splitter2;
    void OnCommand(wxCommandEvent &event);
    void OnUpdateUI(wxUpdateUIEvent &event);
    void OnClose(wxCloseEvent &event);
    void OnNotebookPageClosing(NotebookEvent &event);
    DECLARE_EVENT_TABLE()
};

#endif // !defined(MURDERSPOT_MAINWIN_H)
