// MurderSpot - a free game based on Clue/Cluedo
// Copyright 2005-2009 Christopher Elsby <chrise@chrise.me.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "global.h"

#include "newnet.h"
#include "guidefs.h"
#include "network.h"
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/valtext.h>
#include <wx/button.h>
#include <wx/utils.h>
#include <wx/event.h>

BEGIN_EVENT_TABLE(NetClientDlg, wxDialog)
    EVT_BUTTON(-1, NetClientDlg::OnCommand)
END_EVENT_TABLE()

NetClientDlg::NetClientDlg(wxWindow *parent)
    : wxDialog(parent, -1, wxString(wxT("Connect to Network"))),
      server(wxT("irc.murderspot.chrise.me.uk")), port(wxT("6667")), channel(wxT("#murderspot")),
      nick(wxGetUserId()), username(wxGetUserId()), realname(wxGetUserName())
{
    nick.Replace(wxT(" "), wxT("_"));
    username.Replace(wxT(" "), wxT("_"));
    if(realname.empty())
        realname = nick;
    wxBoxSizer *mainsizer = new wxBoxSizer(wxVERTICAL);
    mainsizer->Add(new wxStaticText(this, -1,
                wxT("This dialog box allows you to connect to a network server to play MurderSpot with other people."
                "\nYour nickname is the name by which you will be known on the server."
                "\nYou may leave the username and real name fields blank if you wish."
                "\nTo connect to a server other than the official one, click Advanced Options.")),
        0, wxALL, CONTROL_SPACING);
    wxFlexGridSizer *settingsizer = new wxFlexGridSizer(2);
    sizer = settingsizer;
    settingsizer->Add(new wxStaticText(this, -1, wxT("Nickname:")), 0, wxALL | wxALIGN_RIGHT, CONTROL_SPACING);
    settingsizer->Add(new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE, &nick)),
        0, wxALL | wxGROW, CONTROL_SPACING);
    settingsizer->Add(new wxStaticText(this, -1, wxT("Username:")), 0, wxALL | wxALIGN_RIGHT, CONTROL_SPACING);
    settingsizer->Add(new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE, &username)),
        0, wxALL | wxGROW, CONTROL_SPACING);
    settingsizer->Add(new wxStaticText(this, -1, wxT("Real name:")), 0, wxALL | wxALIGN_RIGHT, CONTROL_SPACING);
    settingsizer->Add(new wxTextCtrl(this, -1, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE, &realname)),
        0, wxALL | wxGROW, CONTROL_SPACING);
    settingsizer->AddGrowableCol(1);
    mainsizer->Add(settingsizer, 0, wxGROW);
    mainsizer->Add(buttonmore = new wxButton(this, wxID_MORE, wxT("Advanced Options")), 0, wxALL, CONTROL_SPACING);
    wxBoxSizer *buttonsizer = new wxBoxSizer(wxHORIZONTAL);
    wxButton *ok = new wxButton(this, wxID_OK, wxT("&OK"));
    ok->SetDefault();
    buttonsizer->Add(ok, 0, wxALL, CONTROL_SPACING);
    buttonsizer->Add(new wxButton(this, wxID_CANCEL, wxT("&Cancel")), 0, wxALL, CONTROL_SPACING);
    mainsizer->Add(buttonsizer);
    SetSizerAndFit(mainsizer);
}

NetClient *NetClientDlg::runDialog()
{
    if(ShowModal() != wxID_OK)
        return NULL;
    NetClient *net = new NetClient;
    unsigned long pt = 6667;
    if(!port.empty())
        port.ToULong(&pt);
    nick.Replace(wxT(" "), wxT("_"));
    username.Replace(wxT(" "), wxT("_"));
    net->connect(server, pt, nick, username, realname, password);
    net->join(channel);
    return net;
}

void NetClientDlg::OnCommand(wxCommandEvent &event)
{
    switch(event.GetId())
    {
    case wxID_MORE:
        if(buttonmore != NULL)
        {
            buttonmore->Destroy();
            buttonmore = NULL;
            wxWindow *focus;
            sizer->Add(new wxStaticText(this, -1, wxT("Server:")), 0, wxALL | wxALIGN_RIGHT, CONTROL_SPACING);
            sizer->Add(focus = new wxTextCtrl(this, -1, server, wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE, &server)),
                0, wxALL | wxGROW, CONTROL_SPACING);
            sizer->Add(new wxStaticText(this, -1, wxT("Port:")), 0, wxALL | wxALIGN_RIGHT, CONTROL_SPACING);
            sizer->Add(new wxTextCtrl(this, -1, port, wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC, &port)),
                0, wxALL | wxGROW, CONTROL_SPACING);
            sizer->Add(new wxStaticText(this, -1, wxT("Channel:")), 0, wxALL | wxALIGN_RIGHT, CONTROL_SPACING);
            sizer->Add(new wxTextCtrl(this, -1, channel, wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE, &channel)),
                0, wxALL | wxGROW, CONTROL_SPACING);
            sizer->Add(new wxStaticText(this, -1, wxT("Server password:")), 0, wxALL | wxALIGN_RIGHT, CONTROL_SPACING);
            sizer->Add(new wxTextCtrl(this, -1, password, wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE, &password)),
                0, wxALL | wxGROW, CONTROL_SPACING);
            Fit();
            focus->SetFocus();
        }
        break;
    default:
        event.Skip();
    }
}
