// MurderSpot - a free game based on Clue/Cluedo
// Copyright 2005-2009 Christopher Elsby <chrise@chrise.me.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#include "global.h"

#include "settingspage.h"
#include "guidefs.h"
#include "cmdids.h"
#include "mainwin.h"
#include "game.h"
#include "network.h"
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/button.h>
#include <wx/listctrl.h>
#include <wx/textctrl.h>
#include <wx/listbox.h>
#include <wx/filedlg.h>
#include <wx/filename.h>
#include <wx/log.h>
#include <wx/checkbox.h>
#include <wx/choice.h>
#include <wx/arrstr.h>

// SettingsPage
void SettingsPage::Create(MainWin *main, wxWindow *parent)
{
    wxPanel::Create(parent, -1);
    mainwin = main;
}


// WelcomePage
wxString WelcomePage::getType() const
{
    return wxT("WelcomePage");
}

void WelcomePage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(new wxStaticText(this, -1, wxT(
        "Welcome to " PACKAGE_STRING "!\n"
        "\n"
        "MurderSpot comes with ABSOLUTELY NO WARRANTY.\n"
        "You use it entirely at your own risk.\n"
        "\n"
        "MurderSpot enables you to play on a single computer or over the Internet.\n"
        "To begin, choose the type of game you would like to play:"
    )), 0, wxALL, CONTROL_SPACING);
    sizer->Add(new wxButton(this, ID_STARTLOCAL, wxT("Play on this computer")), 0, wxALL, CONTROL_SPACING);
    sizer->Add(new wxButton(this, ID_NETCONNECT, wxT("Play over the Internet")), 0, wxALL, CONTROL_SPACING);
    SetSizerAndFit(sizer);
}


// NetWelcomePage
wxString NetWelcomePage::getType() const
{
    return wxT("NetWelcomePage");
}

void NetWelcomePage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(new wxStaticText(this, -1, wxT(
        "When the connection is established to the server, you will be able to chat with other players in the #murderspot tab below.\n"
        "See Network Status for information regarding your connection to the server.\n"
        "A \"*\" character at the end of a tab name indicates that there is a new message on that tab.\n"
        "If you have any questions about how to play, feel free to ask in the chat.\n"
        "\n"
        "When you are connected, you can choose a game table to set up the game:"
    )), 0, wxALL, CONTROL_SPACING);
    sizer->Add(new wxButton(this, ID_SELECTTABLE, wxT("Select a game table to play on")), 0, wxALL, CONTROL_SPACING);
    sizer->Add(new wxStaticText(this, -1, wxT(
        "Alternatively, you can start a local game with all players on this computer:"
    )), 0, wxALL, CONTROL_SPACING);
    sizer->Add(new wxButton(this, ID_STARTLOCAL, wxT("Start a local game on this computer")), 0, wxALL, CONTROL_SPACING);
    SetSizerAndFit(sizer);
}


// SelectTablePage
wxString SelectTablePage::getType() const
{
    return wxT("SelectTablePage");
}

SelectTablePage::SelectTablePage()
    : listing(FALSE)
{
}

void SelectTablePage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(new wxStaticText(this, -1, wxT(
        "Choose a table to join. Pick an empty table if you want to host your own game."
    )), 0, wxALL, CONTROL_SPACING);
    tablelist = new wxListView(this, -1, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL);
    tablelist->InsertColumn(0, wxT("Table"));
    tablelist->InsertColumn(1, wxT("Status"));
    tablelist->InsertColumn(2, wxT("Host"));
    tablelist->InsertColumn(3, wxT("Players"));
    tablelist->InsertColumn(4, wxT("Description"));
    sizer->Add(tablelist, 1, wxGROW | wxALL, CONTROL_SPACING);
    wxBoxSizer *sizer2 = new wxBoxSizer(wxHORIZONTAL);
    sizer2->Add(new wxStaticText(this, -1, wxT("Description:")), 0, wxALL, CONTROL_SPACING);
    description = new wxTextCtrl(this, -1);
    sizer2->Add(description, 1, wxALL, CONTROL_SPACING);
    sizer2->Add(new wxStaticText(this, -1, wxT("Password:")), 0, wxALL, CONTROL_SPACING);
    password = new wxTextCtrl(this, -1);
    sizer2->Add(password, 1, wxALL, CONTROL_SPACING);
    sizer->Add(sizer2, 0, wxGROW);
    wxBoxSizer *sizer3 = new wxBoxSizer(wxHORIZONTAL);
    sizer3->Add(new wxButton(this, wxID_MORE, wxT("Advanced Settings")), 0, wxALL, CONTROL_SPACING);
    sizer3->Add(channelLabel = new wxStaticText(this, -1, wxT("Channel")), 0, wxALL, CONTROL_SPACING);
    sizer3->Add(channel = new wxTextCtrl(this, -1), 1, wxALL, CONTROL_SPACING);
    sizer3->Add(hostLabel = new wxStaticText(this, -1, wxT("Host")), 0, wxALL, CONTROL_SPACING);
    sizer3->Add(host = new wxTextCtrl(this, -1), 1, wxALL, CONTROL_SPACING);
    sizer->Add(sizer3, 0, wxGROW);
    wxBoxSizer *sizer4 = new wxBoxSizer(wxHORIZONTAL);
    sizer4->Add(new wxButton(this, ID_TABLEGETLIST, wxT("Refresh List")), 0, wxALL, CONTROL_SPACING);
    sizer4->Add(new wxButton(this, ID_TABLEHOST, wxT("Host")), 0, wxALL, CONTROL_SPACING);
    sizer4->Add(new wxButton(this, ID_TABLEJOIN, wxT("Join")), 0, wxALL, CONTROL_SPACING);
    sizer4->Add(new wxButton(this, wxID_CANCEL), 0, wxALL, CONTROL_SPACING);
    sizer->Add(sizer4);
    SetSizerAndFit(sizer);
    channelLabel->Hide();
    channel->Hide();
    hostLabel->Hide();
    host->Hide();
    if(mainwin != NULL)
        mainwin->setTableList(this);
}

SelectTablePage::~SelectTablePage()
{
    if(mainwin != NULL && mainwin->getTableList() == this)
        mainwin->setTableList(NULL);
}

void SelectTablePage::receiveChat(wxString sender, wxString target, wxString type, wxString msg)
{
    // Use LIST replies to populate the table list.
    if(type == wxT("LIST"))
        addTable(msg);
    else if(type == wxT("LISTEND"))
        endTableList();
}

void SelectTablePage::beginTableList()
{
    if(!listing)
    {
        listing = TRUE;
        tablelist->DeleteAllItems();
        channel->SetValue(wxEmptyString);
        host->SetValue(wxEmptyString);
    }
}

void SelectTablePage::addTable(wxString listmsg)
{
    size_t pos, pos2;
    // Get channel name.
    pos = listmsg.find(wxT(" "));
    if(pos >= listmsg.size())
        return;
    wxString chan = listmsg.substr(0, pos);
    listmsg.erase(0, pos + 1);
    // Get table number.
    chan.MakeLower();
    if(chan.substr(0, 17) != wxT("#murderspot-table"))
        return;
    chan.erase(0, 17);
    long id = 0;
    if(!chan.ToLong(&id) || id < 1)
        return;
    wxString strid;
    strid << id;
    if(strid != chan)
        return;
    // Get the number of users.
    pos = listmsg.find(wxT(" "));
    if(pos >= listmsg.size())
        return;
    wxString strnusers = listmsg.substr(0, pos);
    // Skip to the topic.
    // Note: The modes are also in here, so information can
    // potentially be obtained from them.
    pos = listmsg.find(wxT(":"));
    wxString desc;
    wxString host;
    wxString status;
    if(pos < listmsg.size())
    {
        listmsg.erase(0, pos + 1);
        // Read information from the channel topic.
        pos = listmsg.find(wxT(" :::: "));
        if(pos < listmsg.size())
        {
            desc = listmsg.substr(pos + 6);
            listmsg.erase(pos);
        }
        pos = listmsg.find(wxT("Host: "));
        if(pos < listmsg.size())
        {
            pos += 6;
            pos2 = listmsg.find(wxT(" "), pos);
            if(pos2 < listmsg.size())
                host = listmsg.substr(pos, pos2 - pos);
            else
                host = listmsg.substr(pos);
        }
        pos = listmsg.find(wxT("Status: "));
        if(pos < listmsg.size())
        {
            pos += 8;
            pos2 = listmsg.find(wxT(" :: "), pos);
            if(pos2 < listmsg.size())
                status = listmsg.substr(pos, pos2 - pos);
            else
                status = listmsg.substr(pos);
        }
    }
    // Add the entry to the list, erasing the list if this is the first.
    beginTableList();
    long n = tablelist->InsertItem(tablelist->GetItemCount(), strid);
    tablelist->SetItem(n, 1, status);
    tablelist->SetItem(n, 2, host);
    tablelist->SetItem(n, 3, strnusers);
    tablelist->SetItem(n, 4, desc);
    tablelist->SetItemData(n, 1);
}

void SelectTablePage::endTableList()
{
    if(listing)
    {
        listing = FALSE;
        wxString strid;
        long n = 0;
        for(int id = 1; id <= 10; ++id)
        {
            strid.clear();
            strid << id;
            for(n = 0; n < tablelist->GetItemCount(); ++n)
            {
                if(tablelist->GetItemText(n) == strid)
                    break;
            }
            if(n != tablelist->GetItemCount())
                continue;
            n = tablelist->InsertItem(tablelist->GetItemCount(), strid);
            tablelist->SetItem(n, 1, wxT("Free"));
            tablelist->SetItem(n, 2, wxEmptyString);
            tablelist->SetItem(n, 3, wxT("0"));
            tablelist->SetItem(n, 4, wxEmptyString);
            tablelist->SetItemData(n, 2);
        }
    }
}

wxString SelectTablePage::getSelectedChannel() const
{
    long n = tablelist->GetFirstSelected();
    if(n < 0)
        return channel->GetValue();
    return wxT("#murderspot-table") + tablelist->GetItemText(n);
}

bool SelectTablePage::canHost() const
{
    long n = tablelist->GetFirstSelected();
    if(n < 0)
        return !channel->GetValue().empty();
    return tablelist->GetItemData(n) == 2;
}

bool SelectTablePage::canJoin() const
{
    long n = tablelist->GetFirstSelected();
    if(n < 0)
        return !channel->GetValue().empty();
    return tablelist->GetItemData(n) == 1;
}

wxString SelectTablePage::getSelectedHost() const
{
    long n = tablelist->GetFirstSelected();
    if(n < 0)
        return host->GetValue();
    return getListField(n, 2);
}

wxString SelectTablePage::getDescription() const
{
    return description->GetValue();
}

wxString SelectTablePage::getPassword() const
{
    return password->GetValue();
}

wxString SelectTablePage::getSelectedValue(int col) const
{
    long n = tablelist->GetFirstSelected();
    if(n < 0)
        return wxEmptyString;
    return getListField(n, col);
}

wxString SelectTablePage::getListField(int row, int col) const
{
    // Look up the text at row row, column col.
    wxListItem itemspec;
    itemspec.m_itemId = row;
    itemspec.m_col = col;
    itemspec.m_mask = wxLIST_MASK_TEXT;
    if(!tablelist->GetItem(itemspec))
        return wxEmptyString;
    return itemspec.m_text;
}

BEGIN_EVENT_TABLE(SelectTablePage, SettingsPage)
    EVT_BUTTON(-1, SelectTablePage::OnCommand)
    EVT_TEXT(-1, SelectTablePage::OnText)
    EVT_LIST_ITEM_SELECTED(-1, SelectTablePage::OnListItemSelected)
END_EVENT_TABLE()

void SelectTablePage::OnCommand(wxCommandEvent &event)
{
    switch(event.GetId())
    {
    case wxID_MORE:
        channelLabel->Show();
        channel->Show();
        hostLabel->Show();
        host->Show();
        Layout();
        break;
    default:
        event.Skip();
    }
}

void SelectTablePage::OnText(wxCommandEvent &event)
{
    if(event.GetEventObject() == channel)
    {
        long n = tablelist->GetFirstSelected();
        if(n >= 0 && getSelectedChannel() != channel->GetValue())
            tablelist->Select(n, FALSE);
    }
    else if(event.GetEventObject() == host)
    {
        long n = tablelist->GetFirstSelected();
        if(n >= 0 && getSelectedHost() != host->GetValue())
            tablelist->Select(n, FALSE);
    }
    else
    {
        event.Skip();
    }
}

void SelectTablePage::OnListItemSelected(wxListEvent &event)
{
    if(event.GetEventObject() == tablelist)
    {
        channel->SetValue(getSelectedChannel());
        host->SetValue(getSelectedHost());
#if 0
        wxString desc = getSelectedValue(4);
        if(!desc.empty())
            description->SetValue(desc);
#endif
    }
    else
    {
        event.Skip();
    }
}


// GameDataFilePage
wxString GameDataFilePage::getType() const
{
    return wxT("GameDataFilePage");
}

void GameDataFilePage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(new wxStaticText(this, -1, wxT("Welcome to the MurderSpot game setup wizard."
        "\nThis page allows you to select the game board file to load."
        "\nEither select a game type from the list, or browse for the data file to use.")), 0, wxALL, CONTROL_SPACING);
    sizer->Add(listbox = new wxListBox(this, -1), 1, wxALL | wxGROW, CONTROL_SPACING);
    wxBoxSizer *sizer2 = new wxBoxSizer(wxHORIZONTAL);
    sizer2->Add(fileInput = new wxTextCtrl(this, -1), 1, wxALL, CONTROL_SPACING);
    sizer2->Add(new wxButton(this, ID_BROWSE, wxT("B&rowse")), 0, wxALL, CONTROL_SPACING);
    sizer->Add(sizer2, 0, wxGROW);
    wxBoxSizer *sizer3 = new wxBoxSizer(wxHORIZONTAL);
    sizer3->Add(new wxButton(this, wxID_OK), 0, wxALL, CONTROL_SPACING);
    sizer3->Add(new wxButton(this, ID_ENDGAME, wxT("&End Game")), 0, wxALL, CONTROL_SPACING);
    sizer->Add(sizer3, 0, wxGROW);
    SetSizerAndFit(sizer);
    // Initialise data.
    wxFileName filename;
    wxString title;
    wxString f = wxFindFirstFile(wxT("*.murderspot"), wxFILE);
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    while(!f.empty())
    {
        filename = f;
        filename.MakeAbsolute();
        f = filename.GetFullPath();
        title = Board::getTitle(f);
        if(!title.empty())
        {
            listbox->Append(title);
            listfiles.Add(f);
            if(game != NULL && f == fileInput->GetValue())
                listbox->SetSelection(listbox->GetCount() - 1);
        }
        f = wxFindNextFile();
    }
}

BEGIN_EVENT_TABLE(GameDataFilePage, SettingsPage)
    EVT_BUTTON(-1, GameDataFilePage::OnCommand)
    EVT_UPDATE_UI(-1, GameDataFilePage::OnUpdateUI)
    EVT_LISTBOX(-1, GameDataFilePage::OnListBoxSelected)
END_EVENT_TABLE()

void GameDataFilePage::OnCommand(wxCommandEvent &event)
{
    wxString filename;
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    switch(event.GetId())
    {
    case ID_BROWSE:
        filename = wxFileSelector(wxT("Choose data file"), wxT(""), wxT(""), wxT(".murderspot"),
                wxT("MurderSpot Data Files (*.murderspot)|*.murderspot|All Files (*)|*"), wxOPEN | wxFILE_MUST_EXIST, this);
        if(!filename.empty())
        {
            listbox->SetSelection(-1);
            fileInput->SetValue(filename);
        }
        break;
    case wxID_OK:
        filename = fileInput->GetValue();
        if(filename.empty())
        {
            wxLogError(wxT("No data file specified."));
        }
        else if(game != NULL)
        {
            game->getBoard()->unload();
            if(game->getBoard()->load(filename))
            {
                game->reset();
                mainwin->newSettingsPage(new GamePlayersPage);
            }
            else
            {
                wxLogError(wxT("Failed to load \"%s\"."), filename.c_str());
                game->getBoard()->unload();
                game->reset();
            }
        }
        break;
    default:
        event.Skip();
    }
}

void GameDataFilePage::OnUpdateUI(wxUpdateUIEvent &event)
{
    switch(event.GetId())
    {
    case ID_BROWSE:
        break;
    case wxID_OK:
        event.Enable(!fileInput->GetValue().empty());
        break;
    default:
        event.Skip();
    }
}

void GameDataFilePage::OnListBoxSelected(wxCommandEvent &event)
{
    if(event.GetEventObject() == listbox)
    {
        if(listbox->GetSelection() >= 0)
        {
            size_t index = listbox->GetSelection();
            if(index < listfiles.Count())
                fileInput->SetValue(listfiles[listbox->GetSelection()]);
        }
    }
    else
    {
        event.Skip();
    }
}


// GameOptionsPage
wxString GameOptionsPage::getType() const
{
    return wxT("GameOptionsPage");
}

void GameOptionsPage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
    wxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(new wxStaticText(this, -1, wxT("This page allows you to configure various game settings."
        "\nYou can leave the default value for any setting that you are not sure about.")), 0, wxALL, CONTROL_SPACING);
    wxSizer *sizer2 = new wxBoxSizer(wxHORIZONTAL);
    sizer2->Add(keeplog = new wxCheckBox(this, -1, wxT("Full game log")), 0, wxALL, CONTROL_SPACING);
    sizer2->Add(roomwarp = new wxCheckBox(this, -1, wxT("Warp suggested suspect to room")), 0, wxALL, CONTROL_SPACING);
    sizer2->Add(startsuggest = new wxChoice(this, -1), 0, wxALL, CONTROL_SPACING);
    startsuggest->Append(wxT("Must move before suggesting"));
    startsuggest->Append(wxT("Can suggest without moving"));
    startsuggest->Append(wxT("Must move before suggesting unless warped"));
    startsuggest->Append(wxT("Can suggest without moving unless warped"));
    sizer->Add(sizer2, 0, wxGROW);
#if 0
    wxSizer *sizer3 = new wxBoxSizer(wxHORIZONTAL);
    sizer3->Add(new wxStaticText(this, -1, wxT("Allow suggestion without moving:")),
        0, wxTOP | wxLEFT | wxRIGHT, CONTROL_SPACING);
    sizer3->Add(startsuggestwarped = new wxCheckBox(this, -1, wxT("For players moved due to a suggestion")), 1, wxALL, CONTROL_SPACING);
    sizer3->Add(startsuggest = new wxCheckBox(this, -1, wxT("For other players")), 1, wxALL, CONTROL_SPACING);
    sizer->Add(sizer3, 0, wxGROW);
#endif
    wxSizer *sizer3 = new wxBoxSizer(wxHORIZONTAL);
    sizer3->Add(new wxStaticText(this, -1, wxT("Movement points per turn (enter \"1-6\" for a standard die):")),
        0, wxTOP | wxLEFT | wxRIGHT, CONTROL_SPACING);
    sizer3->Add(dice = new wxTextCtrl(this, -1, wxT("")), 1, wxALL, CONTROL_SPACING);
    sizer->Add(sizer3, 0, wxGROW);
    sizer->Add(new wxStaticText(this, -1, wxT("Movement points needed (enter \"-\" to make the action impossible):")),
        0, wxTOP | wxLEFT | wxRIGHT, CONTROL_SPACING);
    wxSizer *sizer4 = new wxBoxSizer(wxHORIZONTAL);
    sizer4->Add(pointlist = new wxListBox(this, -1), 1, wxALL | wxGROW, CONTROL_SPACING);
    wxSizer *sizer5 = new wxBoxSizer(wxVERTICAL);
    sizer5->Add(pointname = new wxStaticText(this, -1, wxEmptyString), 0, wxTOP | wxLEFT | wxRIGHT | wxGROW, CONTROL_SPACING);
    sizer5->Add(points = new wxTextCtrl(this, -1, wxEmptyString), 0, wxALL | wxGROW, CONTROL_SPACING);
    sizer4->Add(sizer5, 1, wxGROW, CONTROL_SPACING);
    sizer->Add(sizer4, 1, wxGROW);
    wxSizer *sizer6 = new wxBoxSizer(wxHORIZONTAL);
    sizer6->Add(new wxButton(this, wxID_OK), 0, wxALL, CONTROL_SPACING);
    sizer6->Add(new wxButton(this, wxID_CANCEL), 0, wxALL, CONTROL_SPACING);
    sizer->Add(sizer6, 0, wxGROW);
    SetSizerAndFit(sizer);
    // Initialise data.
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    if(game != NULL)
    {
        keeplog->SetValue(!game->autoClearLog);
        roomwarp->SetValue(game->roomWarp);
        startsuggest->SetSelection(game->startSuggest ? (game->startSuggestWarped ? 1 : 3) : (game->startSuggestWarped ? 2 : 0));
#if 0
        startsuggest->SetValue(game->startSuggest);
        startsuggestwarped->SetValue(game->startSuggestWarped);
#endif
        dice->SetValue(game->getBoard()->getDiceString());
        pointlist->Clear();
        wxString text;
        for(wxUint8 n = 0; n < game->getBoard()->getTurnConfigCount(); ++n)
        {
            pointvals[n] = game->getBoard()->getTurnConfigValue(n);
            text = game->getBoard()->getTurnConfigName(n);
            if(pointvals[n] < 255)
                text << wxT(" (") << (int) pointvals[n] << wxT(")");
            else
                text << wxT(" (-)");
            pointlist->Append(text);
        }
        pointlist->SetSelection(-1);
        pointname->SetLabel(wxT("Select an action from the left."));
        points->Clear();
        points->Disable();
    }
}

BEGIN_EVENT_TABLE(GameOptionsPage, SettingsPage)
    EVT_BUTTON(-1, GameOptionsPage::OnCommand)
    EVT_UPDATE_UI(-1, GameOptionsPage::OnUpdateUI)
    EVT_LISTBOX(-1, GameOptionsPage::OnSelection)
    EVT_TEXT(-1, GameOptionsPage::OnTextUpdate)
END_EVENT_TABLE()

void GameOptionsPage::OnCommand(wxCommandEvent &event)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    switch(event.GetId())
    {
    case wxID_OK:
        if(game != NULL)
        {
            wxString dicestr = dice->GetValue();
            if(dicestr.empty() || game->getBoard()->setDiceString(dicestr))
            {
                game->roomWarp = roomwarp->GetValue();
                game->autoClearLog = !keeplog->GetValue();
                int sel = startsuggest->GetSelection();
                game->startSuggest = (sel == 1 || sel == 3);
                game->startSuggestWarped = (sel == 1 || sel == 2);
                for(wxUint8 n = 0; n < game->getBoard()->getTurnConfigCount(); ++n)
                    game->getBoard()->setTurnConfigValue(n, pointvals[n]);
                mainwin->newSettingsPage(new GamePlayersPage);
            }
            else
            {
                wxLogError(wxT("Invalid movement point specification.\nPlease either type a single number or a range (e.g. 1-6)."));
            }
        }
        break;
    case wxID_CANCEL:
        if(mainwin != NULL)
            mainwin->newSettingsPage(new GamePlayersPage);
        break;
    default:
        event.Skip();
    }
}

void GameOptionsPage::OnUpdateUI(wxUpdateUIEvent &event)
{
    switch(event.GetId())
    {
    case wxID_CANCEL:
    case wxID_OK:
        break;
    default:
        event.Skip();
    }
}

void GameOptionsPage::OnSelection(wxCommandEvent &event)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    if(event.GetEventObject() == pointlist)
    {
        if(game != NULL)
        {
            int n = pointlist->GetSelection();
            if(n >= 0)
            {
                wxString str;
                if(pointvals[n] < 255)
                    str << (int) pointvals[n];
                else
                    str = wxT("-");
                pointname->SetLabel(game->getBoard()->getTurnConfigName(n) + wxT(":"));
                points->SetValue(str);
                points->Enable();
            }
            else
            {
                pointname->SetLabel(wxT("Select an action from the left."));
                points->Clear();
                points->Disable();
            }
        }
    }
    else
    {
        event.Skip();
    }
}

void GameOptionsPage::OnTextUpdate(wxCommandEvent &event)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    if(event.GetEventObject() == points)
    {
        if(game != NULL)
        {
            int n = pointlist->GetSelection();
            if(n >= 0)
            {
                wxString str = points->GetValue();
                if(!str.empty())
                {
                    if(str == wxT("-"))
                    {
                        pointvals[n] = 255;
                    }
                    else
                    {
                        long value;
                        if(str.ToLong(&value) && value >= 0 && value < 255)
                            pointvals[n] = value;
                    }
                    wxString text = game->getBoard()->getTurnConfigName(n);
                    if(pointvals[n] < 255)
                        text << wxT(" (") << (int) pointvals[n] << wxT(")");
                    else
                        text << wxT(" (-)");
                    pointlist->SetString(n, text);
                }
            }
        }
    }
    else
    {
        event.Skip();
    }
}


// GamePlayersPage
wxString GamePlayersPage::getType() const
{
    return wxT("GamePlayersPage");
}

void GamePlayersPage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    if(game == NULL || !game->started())
        sizer->Add(new wxStaticText(this, -1, wxT("This page allows you to assign human or computer players to each suspect.\n")
                    wxT("First select a suspect in the lefthand list, then select the player in the righthand list.\n")
                    wxT("You must assign players to at least 3 suspects to start the game.")), 0, wxALL, CONTROL_SPACING);
    else
        sizer->Add(new wxStaticText(this, -1, wxT("The game has been paused.\n")
                    wxT("You may now change the player assignments.\n")
                    wxT("Note that you cannot add or remove players.")), 0, wxALL, CONTROL_SPACING);
    wxBoxSizer *sizer2 = new wxBoxSizer(wxHORIZONTAL);
    sizer2->Add(suspectlist = new wxListBox(this, -1), 1, wxALL | wxGROW, CONTROL_SPACING);
    sizer2->Add(playerlist = new wxListBox(this, -1), 1, wxALL | wxGROW, CONTROL_SPACING);
    sizer->Add(sizer2, 1, wxGROW, 0);
    wxBoxSizer *sizer3 = new wxBoxSizer(wxHORIZONTAL);
    sizer3->Add(new wxStaticText(this, -1, wxT("Game description (optional): ")), 0, wxALL, CONTROL_SPACING);
    sizer3->Add(description = new wxTextCtrl(this, -1), 1, wxALL, CONTROL_SPACING);
    sizer->Add(sizer3, 0, wxGROW, 0);
    sizer->Add(new wxButton(this, ID_GAMEOPTIONS, wxT("Game &Options")), 0, wxALL, CONTROL_SPACING);
    wxBoxSizer *sizer4 = new wxBoxSizer(wxHORIZONTAL);
    sizer4->Add(new wxButton(this, wxID_OK), 0, wxALL, CONTROL_SPACING);
    sizer4->Add(new wxButton(this, ID_ENDGAME, wxT("&End Game")), 0, wxALL, CONTROL_SPACING);
    sizer->Add(sizer4, 0, wxGROW);
    SetSizerAndFit(sizer);
    // Initialise data.
    for(wxUint8 suspect = game->getBoard()->suspectMin(); suspect < game->getBoard()->suspectMax(); ++suspect)
        suspectlist->Append(game->getBoard()->getCardName(suspect));
    updateList();
    description->SetValue(game->getDescription());
    if(game != NULL && game->started())
        game->pauseGame();
}

void GamePlayersPage::receiveChat(wxString sender, wxString target, wxString type, wxString msg)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    type.MakeUpper();
    if(game != NULL && !target.empty() && game->getChannel() != NULL && target.Lower() == game->getChannel()->name.Lower() &&
      (type == wxT("JOIN") || type == wxT("PART") || type == wxT("KICK") || type == wxT("QUIT") || type == wxT("NICK")))
        updateList();
}

void GamePlayersPage::updateList()
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    wxArrayString newplayerlist;
    for(wxUint8 type = 0; type < playerTypeCount; ++type)
        newplayerlist.Add(playerTypeNames[type]);
    if(game != NULL)
    {
        if(game->getChannel() != NULL)
        {
            NetUserChannelList::Node *node;
            for(node = game->getChannel()->users.GetFirst(); node != NULL; node = node->GetNext())
            {
                if(node->GetData() != NULL && node->GetData()->getUser() != NULL && node->GetData()->getUser() != game->getChannel()->getUser())
                    newplayerlist.Add(wxT("Network: ") + node->GetData()->getUser()->nick);
            }
        }
        playerlist->Set(newplayerlist);
        updateSelection();
        Player *player;
        wxString str;
        for(wxUint8 suspect = game->getBoard()->suspectMin(); suspect < game->getBoard()->suspectMax(); ++suspect)
        {
            str = game->getBoard()->getCardName(suspect) + wxT(" - ");
            player = game->getPlayer(suspect);
            if(player == NULL)
                str += wxT("None");
            else
            {
                if(!player->name.empty())
                    str += player->name + wxT(" (");
                if(player->netuser == NULL)
                    str += playerTypeNames[player->type];
                else
                    str += wxT("Network: ") + player->netuser->nick;
                if(!player->name.empty())
                    str += wxT(")");
            }
            suspectlist->SetString(suspect, str);
        }
    }
}

void GamePlayersPage::updateSelection()
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    if(game != NULL)
    {
        Player *player = game->getPlayer(suspectlist->GetSelection() + game->getBoard()->suspectMin());
        if(player == NULL)
        {
            playerlist->SetSelection(0);
        }
        else
        {
            if(player->netuser == NULL)
                playerlist->SetSelection(player->type);
            else
                playerlist->SetStringSelection(wxT("Network: ") + player->netuser->nick);
        }
    }
}

BEGIN_EVENT_TABLE(GamePlayersPage, SettingsPage)
    EVT_BUTTON(-1, GamePlayersPage::OnCommand)
    EVT_UPDATE_UI(-1, GamePlayersPage::OnUpdateUI)
    EVT_LISTBOX(-1, GamePlayersPage::OnListBoxSelected)
    EVT_TEXT(-1, GamePlayersPage::OnText)
END_EVENT_TABLE()

void GamePlayersPage::OnCommand(wxCommandEvent &event)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    switch(event.GetId())
    {
    case ID_GAMEOPTIONS:
        if(mainwin != NULL)
            mainwin->newSettingsPage(new GameOptionsPage);
        break;
    case wxID_OK:
        if(game != NULL)
        {
            ++mainwin->alertBlock;
            game->setDescription(description->GetValue(), FALSE);
            int playerCount = game->getPlayerCount();
            if(playerCount < 3)
            {
                wxLogError(wxT("3 players are needed to start the game, but you have only added %i."), playerCount);
            }
            else if(game->started())
            {
                game->netSendCards();
                game->resumeGame();
                game->update();
                mainwin->newSettingsPage(NULL);
            }
            else
            {
                if(game->startGame())
                {
                    game->update();
                    mainwin->updateImage();
                    mainwin->newSettingsPage(NULL);
                }
                else
                {
                    wxLogError(wxT("Failed to start game."));
                }
            }
            --mainwin->alertBlock;
        }
        break;
    default:
        event.Skip();
    }
}

void GamePlayersPage::OnUpdateUI(wxUpdateUIEvent &event)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    switch(event.GetId())
    {
    case ID_GAMEOPTIONS:
        break;
    case wxID_OK:
        event.Enable(game == NULL || game->getPlayerCount() >= 3);
        break;
    default:
        event.Skip();
    }
}

void GamePlayersPage::OnListBoxSelected(wxCommandEvent &event)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    if(game == NULL)
    {
    }
    else if(event.GetEventObject() == suspectlist)
    {
        updateSelection();
    }
    else if(event.GetEventObject() == playerlist)
    {
        Player *player = game->getPlayer(suspectlist->GetSelection() + game->getBoard()->suspectMin());
        if(player != NULL)
        {
            int sel = playerlist->GetSelection();
            if(game->started() && (sel == 0) != (player->type == 0))
            {
                updateSelection();
            }
            else if(sel >= 0)
            {
                if(sel < playerTypeCount)
                {
                    player->type = sel;
                    player->netuser = NULL;
                }
                else if(game->getChannel() != NULL)
                {
                    player->type = 1;
                    player->netuser = game->getChannel()->getUser(playerlist->GetStringSelection().substr(9));
                }
                wxString str = game->getBoard()->getCardName(suspectlist->GetSelection() + game->getBoard()->suspectMin()) + wxT(" - ");
                if(!player->name.empty())
                    str += player->name + wxT(" (");
                str += playerlist->GetStringSelection();
                if(!player->name.empty())
                    str += wxT(")");
                suspectlist->SetString(suspectlist->GetSelection(), str);
            }
        }
    }
    else
    {
        event.Skip();
    }
}

void GamePlayersPage::OnText(wxCommandEvent &event)
{
    Game *game = NULL;
    if(mainwin != NULL)
        game = mainwin->getGame();
    Player *player = NULL;
    if(game != NULL)
        player = game->getPlayer(suspectlist->GetSelection() + game->getBoard()->suspectMin());
    if(player != NULL)
    {
        wxString str = game->getBoard()->getCardName(suspectlist->GetSelection() + game->getBoard()->suspectMin()) + wxT(" - ");
        if(!player->name.empty())
            str += player->name + wxT(" (");
        str += playerlist->GetStringSelection();
        if(!player->name.empty())
            str += wxT(")");
        suspectlist->SetString(suspectlist->GetSelection(), str);
    }
}


// WaitForHostPage
wxString WaitForHostPage::getType() const
{
    return wxT("WaitForHostPage");
}

void WaitForHostPage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(0, 0, 1);
    sizer->Add(new wxStaticText(this, -1, wxT("Please wait for the host to start the game...")), 0, wxALL | wxALIGN_CENTER, 5);
    sizer->Add(new wxButton(this, ID_ENDGAME, wxT("Leave Game")), 0, wxALL | wxALIGN_CENTER, 5);
    sizer->Add(0, 0, 1);
    SetSizerAndFit(sizer);
}


#if 0 // Template for adding a new page.
// SomeOtherPage
wxString SomeOtherPage::getType() const
{
    return wxT("SomeOtherPage");
}

void SomeOtherPage::Create(MainWin *main, wxWindow *parent)
{
    SettingsPage::Create(main, parent);
}
#endif

