// MurderSpot - a free game based on Clue/Cluedo
// Copyright 2005-2009 Christopher Elsby <chrise@chrise.me.uk>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#ifndef MURDERSPOT_SETTINGSPAGE_H
#define MURDERSPOT_SETTINGSPAGE_H

#include <wx/panel.h>
#include <wx/string.h>
#include <wx/event.h>

class MainWin;
class wxListView;
class wxTextCtrl;
class wxCommandEvent;
class wxListEvent;
class wxListBox;
class wxCheckBox;
class wxChoice;
class wxStaticText;

class SettingsPage : public wxPanel
{
public:
    virtual wxString getType() const = 0;
    virtual void Create(MainWin *main, wxWindow *parent);
    virtual void receiveChat(wxString sender, wxString target, wxString type, wxString msg) { }
    MainWin *mainwin;
};

class WelcomePage : public SettingsPage
{
public:
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
};

class NetWelcomePage : public SettingsPage
{
public:
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
};

class SelectTablePage : public SettingsPage
{
public:
    SelectTablePage();
    virtual ~SelectTablePage();
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
    void beginTableList();
    void addTable(wxString listmsg);
    void endTableList();
    virtual void receiveChat(wxString sender, wxString target, wxString type, wxString msg);
    wxString getSelectedChannel() const;
    bool canHost() const;
    bool canJoin() const;
    wxString getSelectedHost() const;
    wxString getDescription() const;
    wxString getPassword() const;
private:
    wxString getSelectedValue(int col) const;
    wxString getListField(int row, int col) const;
    void OnCommand(wxCommandEvent &event);
    void OnText(wxCommandEvent &event);
    void OnListItemSelected(wxListEvent &event);
    bool listing;
    wxListView *tablelist;
    wxTextCtrl *description;
    wxTextCtrl *password;
    wxStaticText *channelLabel;
    wxTextCtrl *channel;
    wxStaticText *hostLabel;
    wxTextCtrl *host;
    DECLARE_EVENT_TABLE()
};

class GameDataFilePage : public SettingsPage
{
public:
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
private:
    void OnCommand(wxCommandEvent &event);
    void OnUpdateUI(wxUpdateUIEvent &event);
    void OnListBoxSelected(wxCommandEvent &event);
    // List of files available.
    wxListBox *listbox;
    wxArrayString listfiles;
    // File to load.
    wxTextCtrl *fileInput;
    DECLARE_EVENT_TABLE()
};

class GameOptionsPage : public SettingsPage
{
public:
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
private:
    void OnCommand(wxCommandEvent &event);
    void OnUpdateUI(wxUpdateUIEvent &event);
    void OnSelection(wxCommandEvent &event);
    void OnTextUpdate(wxCommandEvent &event);
    // Whether the suspect named in a suggestion should be moved to the room.
    wxCheckBox *roomwarp;
    // Whether the game log should be kept for the whole game.
    wxCheckBox *keeplog;
    // Whether a player needs to move rooms before suggesting.
    wxChoice *startsuggest;
    // Text box for dice settings.
    wxTextCtrl *dice;
    // Configurable movement points.
    wxListBox *pointlist;
    wxStaticText *pointname;
    wxTextCtrl *points;
    wxUint8 pointvals[255];
    DECLARE_EVENT_TABLE()
};

class GamePlayersPage : public SettingsPage
{
public:
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
    virtual void receiveChat(wxString sender, wxString target, wxString type, wxString msg);
    void updateList();
private:
    void updateSelection();
    void OnCommand(wxCommandEvent &event);
    void OnUpdateUI(wxUpdateUIEvent &event);
    void OnListBoxSelected(wxCommandEvent &event);
    void OnText(wxCommandEvent &event);
    // List of suspects.
    wxListBox *suspectlist;
    // List of players.
    wxListBox *playerlist;
    // Game description.
    wxTextCtrl *description;
    DECLARE_EVENT_TABLE()
};

class WaitForHostPage : public SettingsPage
{
public:
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
};

#if 0 // Template for adding a new page.
class SomeOtherPage : public SettingsPage
{
public:
    virtual wxString getType() const;
    virtual void Create(MainWin *main, wxWindow *parent);
};
#endif

#endif // !defined(MURDERSPOT_SETTINGSPAGE_H)
